namespace BitBadger.Documents.Tests.CSharp;

public class NumIdDocument
{
    public int Key { get; set; } = 0;
    public string Text { get; set; } = "";
}

public class SubDocument
{
    public string Foo { get; set; } = "";
    public string Bar { get; set; } = "";
}

public class JsonDocument
{
    public string Id { get; set; } = "";
    public string Value { get; set; } = "";
    public int NumValue { get; set; } = 0;
    public SubDocument? Sub { get; set; } = null;

    /// <summary>
    /// A set of documents used for integration tests
    /// </summary>
    public static readonly List<JsonDocument> TestDocuments =
    [
        new() { Id = "one", Value = "FIRST!", NumValue = 0 },
        new() { Id = "two", Value = "another", NumValue = 10, Sub = new() { Foo = "green", Bar = "blue" } },
        new() { Id = "three", Value = "", NumValue = 4 },
        new() { Id = "four", Value = "purple", NumValue = 17, Sub = new() { Foo = "green", Bar = "red" } },
        new() { Id = "five", Value = "purple", NumValue = 18 }
    ];

    /// <summary>The JSON for document ID `one`</summary>
    public static string One = """{"Id":"one","Value":"FIRST!","NumValue":0,"Sub":null}""";

    /// <summary>The JSON for document ID `two`</summary>
    public static string Two = """{"Id":"two","Value":"another","NumValue":10,"Sub":{"Foo":"green","Bar":"blue"}}""";

    /// <summary>The JSON for document ID `three`</summary>
    public static string Three = """{"Id":"three","Value":"","NumValue":4,"Sub":null}""";

    /// <summary>The JSON for document ID `four`</summary>
    public static string Four = """{"Id":"four","Value":"purple","NumValue":17,"Sub":{"Foo":"green","Bar":"red"}}""";

    /// <summary>The JSON for document ID `five`</summary>
    public static string Five = """{"Id":"five","Value":"purple","NumValue":18,"Sub":null}""";

}

public class ArrayDocument
{
    public string Id { get; set; } = "";
    public string[] Values { get; set; } = [];

    /// <summary>
    /// A set of documents used for integration tests
    /// </summary>
    public static readonly List<ArrayDocument> TestDocuments =
    [
        new() { Id = "first", Values = ["a", "b", "c"] },
        new() { Id = "second", Values = ["c", "d", "e"] },
        new() { Id = "third", Values = ["x", "y", "z"] }
    ];
}
